#!/bin/bash -e

usage="
Usage: $0 [OPTIONS]
Installs OpenNI to current machine.

-i,--install
    Installs OpenNI (default mode)
-u,--uninstall
    Uninstalls OpenNI.
-n,--install-nite
    install nite
-c,--cross-compile-rootfs <path>
    Used for cross-compiling. Installs OpenNI to <path> instead of '/'.
-h,--help
    Shows this help screen.
"

OS_NAME=`uname -s`

if [ "`uname -s`" == "Darwin" ]; then
        LIBEXT="dylib"
else
        LIBEXT="so"
fi

case $OS_NAME in
Darwin)
    MODULES="libnimMockNodes.dylib libnimCodecs.dylib libnimRecorder.dylib"
    GACDIR=""
    ;;
*)
    MODULES="libnimMockNodes.so libnimCodecs.so libnimRecorder.so"
    GACDIR="-gacdir $rootfs/usr"
    ;;
esac

TERABEE_MODULES="libmodule-terabee.$LIBEXT"

SCRIPT_DIR=`pwd`/`dirname $0`

# detect Linux OS distribution
printf "Detect OS distribution version and create links\n"
DIST_ID=`lsb_release -i -s`
DIST_RE=`lsb_release -r -s`
DIST_CO=`lsb_release -c -s`
if [ "$DIST_ID" == "Ubuntu" ]; then
    if [ "$DIST_RE" == "16.04" ] || [ "$DIST_RE" == "16.10" ] || [ "$DIST_RE" == "17.04" ] || [ "$DIST_RE" == "17.10" ]; then
        DIST_REL_NAME=16.04
    else
        DIST_REL_NAME=14.04
    fi
    printf "It's $DIST_ID $DIST_RE\n\n"
elif [ "$DIST_ID" == "LinuxMint" ]; then
    if [ "$DIST_RE" == "17" ]; then
        DIST_REL_NAME=14.04
    elif echo "$DIST_RE" | grep -q "17.*"; then
        DIST_REL_NAME=14.04
    else
        DIST_REL_NAME=16.04
    fi
    printf "It's $DIST_ID $DIST_RE\n\n"
elif [ "$DIST_ID" == "elementary" ]; then
    if [ "$DIST_RE" == "0.3" ]; then
        DIST_REL_NAME=14.04
    elif echo "$DIST_RE" | grep -q "0.3.*"; then
        DIST_REL_NAME=14.04
    else
        DIST_REL_NAME=16.04
    fi
    printf "It's $DIST_ID $DIST_RE\n\n"
elif [ "$DIST_ID" == "Debian" ] || [ "$DIST_ID" = "Raspbian" ]; then
    if [ "$DIST_CO" == "stretch" ]; then
        DIST_REL_NAME=16.04
    else
        DIST_REL_NAME=14.04
    fi
else
    #Let Ubuntu 16.04 as default distribution for install
    printf "Unknown, try default Ubutnu 16.04 build\n\n"
    DIST_REL_NAME=16.04
fi

ln -sf $SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libbilateralfilter.so $SCRIPT_DIR/TERABEE/TERABEE/lib/libbilateralfilter.so
ln -sf $SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libdevicesensor.so $SCRIPT_DIR/TERABEE/TERABEE/lib/libdevicesensor.so
if [ -f "$SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libclproc.so" ]; then
    ln -sf $SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libclproc.so $SCRIPT_DIR/TERABEE/TERABEE/lib/libclproc.so
fi
if [ -f "$SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libcudaproc.so" ]; then
    ln -sf $SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libcudaproc.so $SCRIPT_DIR/TERABEE/TERABEE/lib/libcudaproc.so
fi
ln -sf $SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/libmodule-terabee.so $SCRIPT_DIR/TERABEE/TERABEE/lib/libmodule-terabee.so

# create file list
LIB_FILES=`ls $SCRIPT_DIR/Lib/*`
BIN_FILES=`ls $SCRIPT_DIR/Bin/ni*`
JAR_FILES=`ls $SCRIPT_DIR/Jar/*.jar`
TERABEE_LIBS=`ls $SCRIPT_DIR/TERABEE/TERABEE/lib/$DIST_REL_NAME/lib*.so*`
TERABEE_CONF=`ls $SCRIPT_DIR/TERABEE/TERABEE/config/*.json`
TERABEE_UDEV=`ls $SCRIPT_DIR/TERABEE/udev/99-TERABEE-video.rules`

rootfs=
install_nite=

# parse command line
while [ "$1" ]; do
    case $1 in
    -i|--install)
        install=yes
        ;;
    -u|--uninstall)
        uninstall=yes
        ;;
    -n|--install_nite)
        install_nite=yes
        ;;
    -c|--cross-staging-dir)
        shift
        rootfs=$1
        ;;
    -h|--help)
        echo "$usage"
        exit 0
        ;;
    *)
        echo "Unrecognized option $1"
        exit 1
    esac
    shift
done

# default mode is install
if [ ! "$install" = yes ] && [ ! "$uninstall" = yes ]; then
    install=yes
fi

# validity check
if [ "$install" = yes ] && [ "$uninstall" = yes ]; then
    echo "-i and -u flags cannot be used together!"
    exit 1
fi

INSTALL_LIB=$rootfs/usr/lib
INSTALL_BIN=$rootfs/usr/bin
INSTALL_TERABEE=$rootfs/usr/etc/TERABEE
INSTALL_TERABEE_LIB=$INSTALL_TERABEE/lib
INSTALL_INC=$rootfs/usr/include/ni
INSTALL_VAR=$rootfs/var/lib/ni
INSTALL_JAR=$rootfs/usr/share/java
INSTALL_UDEV=$rootfs/etc/udev/rules.d

# make all calls into OpenNI run in this filesystem
export OPEN_NI_INSTALL_PATH=$rootfs
# make sure the staging dir OpenNI is the one being run
export LD_LIBRARY_PATH=$INSTALL_LIB

if [ "$install" = yes ]; then
    printf "Installing OpenNI\n"
    printf "****************************\n\n"

    # copy libraries
    printf "copying shared libraries..."
    cp $LIB_FILES $INSTALL_LIB
    printf "OK\n"

    # utilities
    printf "copying executables..."
    cp $BIN_FILES $INSTALL_BIN
    printf "OK\n"

    # include files
    printf "copying include files..."
    mkdir -p $INSTALL_INC
    cp -r Include/* $INSTALL_INC
    printf "OK\n"

    # TERABEE libs
    printf "copying TERABEE libraries..."
	mkdir -p $INSTALL_TERABEE_LIB
    for filename in $TERABEE_LIBS; do
        if [ -f $filename ]; then
            cp -p $filename $INSTALL_TERABEE_LIB/
            cp -p $filename $INSTALL_LIB/
        else
            # remember to remove non-existent files from system
            [ -f $INSTALL_TERABEE_LIB/`basename $filename` ] && rm -f $INSTALL_TERABEE_LIB/`basename $filename`
            [ -f $INSTALL_LIB/`basename $filename` ] && rm -f $INSTALL_LIB/`basename $filename`
        fi
    done

    printf "OK\n"

    # TERABEE config
    printf "copying TERABEE config..."
    cp -p $TERABEE_CONF $INSTALL_TERABEE_LIB/
    printf "OK\n"

    # TERABEE udev rules
    printf "copying TERABEE udev rules..."
    cp $TERABEE_UDEV $INSTALL_UDEV
    printf "OK\n"

    # create database dir
    printf "creating database directory..."
    mkdir -p $INSTALL_VAR
    printf "OK\n"

    # register modules
    for module in $MODULES; do
        printf "registering module '$module'..."
        $INSTALL_BIN/niReg -r $INSTALL_LIB/$module
        printf "OK\n"
    done

    # register TERABEE module
    for module in $TERABEE_MODULES; do
        printf "registering module '$module'..."
        $INSTALL_BIN/niReg -r $INSTALL_TERABEE_LIB/$module
    done
    printf "OK\n"

    # mono
    #if [ -f $rootfs/usr/bin/gmcs -a -f Bin/OpenNI.net.dll ]; then
    #	printf "Installing .Net wrappers...\n"
    #	gacutil -i Bin/OpenNI.net.dll -package 2.0 $GACDIR
    #fi

    # java wrappers
    printf "creating java bindings directory..."
    mkdir -p $INSTALL_JAR
    printf "OK\n"

    printf "Installing java bindings..."
    cp $JAR_FILES $INSTALL_JAR
    printf "OK\n"

if [ "$install_nite" = yes ]; then
    # nite
    printf "\nInstalling NITE\n"
    printf "***************\n"
    printf "Copying shared libraries... "
    cp Nite/Bin/libXnVNite*$LIBEXT $rootfs/usr/lib
    cp Nite/Bin/libXnVCNITE*$LIBEXT $rootfs/usr/lib
    cp Nite/Bin/libXnVNITE.jni*$LIBEXT $rootfs/usr/lib
    printf "OK\n"

#    DIRS=`ls -l  | egrep '^d' | awk '{print $8}'`
    for fdir in `ls -1 ./Nite | grep Features`
    do
        printf "Installing module '$fdir'...\n"
        mkdir -p $rootfs/usr/etc/primesense/$fdir
        cp -r Nite/$fdir/Data/* $rootfs/usr/etc/primesense/$fdir
        for so in `ls -1 Nite/$fdir/Bin/lib*$LIBEXT`
        do
            base=`basename $so`
            printf "Registering module '$base'... "
            cp $so $rootfs/usr/lib
            $INSTALL_BIN/niReg -r $rootfs/usr/lib/$base $rootfs/usr/etc/primesense/$fdir
            printf "OK\n"
        done
        for bin in `ls -1 Nite/$fdir/Bin | grep XnVSceneServer`
        do
            printf "Copying XnVSceneServer... "
            full=Nite/$fdir/Bin/$bin
            cp $full $rootfs/usr/bin
            chmod +x $rootfs/usr/bin/$bin
            printf "OK\n"
        done
    done

    for hdir in `ls -1 ./Nite | grep Hands`
    do
        printf "Installing module '$fdir'\n"
        mkdir -p $rootfs/usr/etc/primesense/$hdir
        cp -r Nite/$hdir/Data/* $rootfs/usr/etc/primesense/$hdir
        for so in `ls -1 Nite/$hdir/Bin/lib*$LIBEXT`
        do
            base=`basename $so`
            printf "registering module '$base'..."
            cp $so $rootfs/usr/lib
            $INSTALL_BIN/niReg -r $rootfs/usr/lib/$base $rootfs/usr/etc/primesense/$hdir
            printf "OK\n"
        done
    done
fi

printf "\n*** DONE ***\n\n"

elif [ "$uninstall" = yes ]; then
    printf "Uninstalling OpenNI\n"
    printf "****************************\n\n"

    # unregister modules
    for module in $MODULES; do
        printf "unregistering module '$module'..."
        if $INSTALL_BIN/niReg -u $INSTALL_LIB/$module; then
            printf "OK\n"
        fi
    done

    # TERABEE module
    for module in $TERABEE_MODULES; do
        printf "unregistering module '$module'..."
        if $INSTALL_BIN/niReg -u $INSTALL_TERABEE_LIB/$module; then
            printf "OK\n"
        fi
    done

    # include files
    printf "removing include files..."
    rm -rf $INSTALL_INC
    printf "OK\n"

    for fmod in `ls -1 $rootfs/usr/lib/libXnVFeatures*$LIBEXT 2>/dev/null || true`
    do
        $INSTALL_BIN/niReg -u $fmod
    done
    for hmod in `ls -1 $rootfs/usr/lib/libXnVHandGenerator*$LIBEXT > /dev/null 2>&1 || true`
    do
        $INSTALL_BIN/niReg -u $hmod
    done

    rm $rootfs/usr/lib/libXnVNite*$LIBEXT 2>/dev/null  || true
    rm $rootfs/usr/lib/libXnVCNITE*$LIBEXT 2>/dev/null  || true
    rm $rootfs/usr/lib/libXnVNITE.jni*$LIBEXT  2>/dev/null  || true
    rm $rootfs/usr/lib/libXnVFeatures*$LIBEXT  2>/dev/null  || true
    rm $rootfs/usr/lib/libXnVHandGenerator*$LIBEXT 2>/dev/null  || true
    rm -rf $rootfs/usr/bin/XnVSceneServer*  2>/dev/null  || true
    rm -rf $rootfs/usr/etc/primesense/Features*  2>/dev/null  || true
    rm -rf $rootfs/usr/etc/primesense/Hands* 2>/dev/null  || true

    # binaries
    printf "removing executables..."
    for filename in $BIN_FILES; do
        rm -f $INSTALL_BIN/`basename $filename`
    done
    printf "OK\n"

    # libraries
    printf "removing shared libraries..."
    for filename in $LIB_FILES; do
        rm -f $INSTALL_LIB/`basename $filename`
    done
    printf "OK\n"

    # mono
    #if [ -f $rootfs/usr/bin/gmcs -a -f Bin/OpenNI.net.dll ]; then
    #	printf "Removing .Net wrappers...\n"
    #	gacutil -u OpenNI.net $GACDIR
    #fi

    # java
    printf "removing java bindings..."
    for filename in $JAR_FILES; do
        rm -f $INSTALL_JAR/`basename $filename`
    done
    printf "OK\n"

    # remove TERABEE libs & config
    printf "removing TERABEE libraries..."
    for filename in $TERABEE_LIBS; do
        [ -f $INSTALL_TERABEE_LIB/`basename $filename` ] && rm -f $INSTALL_TERABEE_LIB/`basename $filename`
        [ -f $INSTALL_LIB/`basename $filename` ] && rm -f $INSTALL_LIB/`basename $filename`
    done
    printf "OK\n"
    printf "removing TERABEE config..."
    for configname in $TERABEE_CONF; do
        rm -f $INSTALL_TERABEE_LIB/`basename $configname`
    done

    rm -rf $INSTALL_TERABEE
    rm -f $INSTALL_UDEV/99-TERABEE-video.rules

    printf "OK\n"

    printf "\n*** DONE ***\n\n"

fi

